/**
 * Convert leading Tabs to Spaces or leading Spaces to Tabs
 *
 * @category  WeBuilder Plugin
 * @package   Convert Leading
 * @author    Peter Klein <pmk@io.dk>
 * @copyright 2017 Peter Klein
 * @license   http://www.freebsd.org/copyright/license.html  BSD License
 * @version   1.01
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *     32   function TabsToSpaces(Sender)
 *     73   function SpacesToTabs(Sender)
 *    126   function RepeatChar(character, repeatCount)
 *    139   function OnInstalled()
 *
 * TOTAL FUNCTIONS: 4
 * (This index is automatically created/updated by the WeBuilder plugin "DocBlock Comments")
 *
 */

/**
 * Convert leading Tabs into Spaces
 *
 * @param  object   Sender
 *
 * @return void
 */
function TabsToSpaces(Sender) {

	Editor.BeginEditing;

	var SL = new TStringList;

	// Save selection/scroll position
	var sel = Editor.Selection,
		topLine = Editor.TopLine,
		startLine = sel.SelStartLine,
		startCol = sel.SelStartCol,
		endLine = sel.SelEndLine,
		endCol = sel.SelEndCol;

	for (var i=0;i<Editor.LineCount;i++) {
		// Using LinesAsDisplayed, all Tabs are converted to Spaces
		// So no additional conversion is needed
		SL.Add(Editor.LinesAsDisplayed[i]);
	}

	Editor.Text = SL.Text;
	delete SL;

	// Restore selection/scroll position
	sel.SelStartLine = startLine;
	sel.SelStartCol = startCol;
	sel.SelEndLine = endLine;
	sel.SelEndCol = endCol;
	Editor.Selection = sel;
	Editor.TopLine = topLine;

	Editor.EndEditing;
}

/**
 * Convert leading Spaces into Tabs
 *
 * @param  object   Sender
 *
 * @return void
 */
function SpacesToTabs(Sender) {

	var tabSize = Script.Settings.TabSize;

	Editor.BeginEditing;

	var SL = new TStringList;

	// Save selection/scroll position
	var sel = Editor.Selection,
		topLine = Editor.TopLine,
		startLine = sel.SelStartLine,
		startCol = sel.SelStartCol,
		endLine = sel.SelEndLine,
		endCol = sel.SelEndCol;

	for (var i=0;i<Editor.LineCount;i++) {
		// RegexMatchAll is buggy and doesn't match empty lines as expected
		// So an additional check for empty strings is required
		if (Editor.LinesAsDisplayed[i] == "") {
			SL.Add(Editor.LinesAsDisplayed[i]);
		}
		else if (RegexMatchAll(Editor.LinesAsDisplayed[i], "^(\\s*)(.*)$", true, match, p) == true) {
			// Find any leading spaces.
			// Using LinesAsDisplayed, all Tabs are converted to Spaces
			var len = Length(_v(match, [0, 1]));
			var remainingSpaces = len - Round(len/tabSize)*tabSize;
			SL.Add(RepeatChar(chr(9), Round(len/tabSize)) + RepeatChar(" ", remainingSpaces) + _v(match, [0, 2]));
		}
	}

	Editor.Text = SL.Text;
	delete SL;

	// Restore selection/scroll position
	sel.SelStartLine = startLine;
	sel.SelStartCol = startCol;
	sel.SelEndLine = endLine;
	sel.SelEndCol = endCol;
	Editor.Selection = sel;
	Editor.TopLine = topLine;

	Editor.EndEditing;
}

/**
 * Create string of repeated characters
 *
 * @param  string   character the character to repeat
 * @param  integer   repeatCount how many times to repeat
 *
 * @return string
 */
function RepeatChar(character, repeatCount) {
	var res = "";
	for (var i=0;i<repeatCount;i++) {
		res += character;
	}
	return res;
}

/**
 * Show info when plugin is installed
 *
 * @return void
 */
function OnInstalled() {
	alert("Convert Leading 1.01 by Peter Klein installed sucessfully!");
}

Script.ConnectSignal("installed", "OnInstalled");
var bmp = new TBitmap;
var icon1 = LoadFileToBitmap(Script.Path + "totab_icon.png", bmp);
var act1 = Script.RegisterDocumentAction("Convert Leading", "Spaces to Tabs", "", "SpacesToTabs");
Actions.SetIcon(act1, bmp);

var icon2 = LoadFileToBitmap(Script.Path + "tospace_icon.png", bmp);
var act2 = Script.RegisterDocumentAction("Convert Leading", "Tabs to Spaces", "", "TabsToSpaces");
Actions.SetIcon(act2, bmp);
delete bmp;
